@@ -1,7 +1,7 @@
 /*
  * gamelist.c -- Functions to manage a gamelist
  *
- * Copyright 1995, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+ * Copyright 1995, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -392,7 +392,7 @@ GameListBuild (FILE *f)
 	}
     }
   }
-GetTimeMark(&t2);printf("GameListBuild %ld msec\n", SubtractTimeMarks(&t2,&t));
+    if(appData.debugMode) { GetTimeMark(&t2);printf("GameListBuild %ld msec\n", SubtractTimeMarks(&t2,&t)); }
     quickFlag = 0;
     PackGame(boards[scratch]); // for appending end-of-game marker.
     DisplayTitle("WinBoard");
@@ -672,4 +672,3 @@ GLT_ParseList ()
 	idx++;
     } while( id != '\0' );
 }
-
