@@ -5,6 +5,8 @@
  *
  * Copyright 2006 Alessandro Scotti
  *
+ * Enhancement Copyright 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
+ *
  * ------------------------------------------------------------------------
  *
  * GNU XBoard is free software: you can redistribute it and/or modify
@@ -18,7 +20,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see http://www.gnu.org/licenses/. 
+ * along with this program. If not, see http://www.gnu.org/licenses/.
  *
  * ------------------------------------------------------------------------
  */
@@ -29,9 +31,10 @@
 
 #include "common.h"
 #include "backend.h"
-Boolean GetArgValue(char *a);				
+Boolean GetArgValue(char *a);
 
-void InitEngineUCI( const char * iniDir, ChessProgramState * cps )
+void
+InitEngineUCI (const char *iniDir, ChessProgramState *cps)
 {   // replace engine command line by adapter command with expanded meta-symbols
     if( cps->isUCI ) {
         char *p, *q;
